/* Author: tmouse a!t bigfoot d!o!t com */

#include <windows.h>

/* 
 * This function enables or disables a privilege for the current process.
 * Common privileges include SE_SHUTDOWN_NAME, SE_BACKUP_NAME, SE_SYSTEMTIME_NAME, etc.
 * This function dynamically loads the required functions so as not to cause load
 * problems on 9x where these functions may not be available.
 */
BOOL WINAPI EnableProcessPrivilege(LPCTSTR lpPrivilege, BOOL bEnable) 
{
	typedef BOOL (WINAPI * fnptr_openprocesstoken)(HANDLE,DWORD,PHANDLE);
	typedef BOOL (WINAPI * fnptr_adjusttokenprivileges)(HANDLE,BOOL,PTOKEN_PRIVILEGES,DWORD,PTOKEN_PRIVILEGES,PDWORD);
	typedef BOOL (WINAPI * fnptr_lookupprivilegevalue)(LPCTSTR,LPCTSTR,PLUID);

	TOKEN_PRIVILEGES tp       = { 0 };
	HMODULE          hAdvapi  = NULL;
	HANDLE           hToken   = NULL;
	BOOL             bResult  = FALSE;
	fnptr_openprocesstoken      openprocesstoken;
	fnptr_adjusttokenprivileges adjusttokenprivileges;
	fnptr_lookupprivilegevalue  lookupprivilegevalue;

	/* Load the required functions */
	if (!(hAdvapi = LoadLibrary(TEXT("Advapi32.dll"))) ||
	    !(openprocesstoken      = (fnptr_openprocesstoken)      GetProcAddress(hAdvapi, "OpenProcessToken"))      ||
	    !(adjusttokenprivileges = (fnptr_adjusttokenprivileges) GetProcAddress(hAdvapi, "AdjustTokenPrivileges")) ||
#ifndef UNICODE
	    !(lookupprivilegevalue  = (fnptr_lookupprivilegevalue)  GetProcAddress(hAdvapi, "LookupPrivilegeValueA")))
#else
	    !(lookupprivilegevalue  = (fnptr_lookupprivilegevalue)  GetProcAddress(hAdvapi, "LookupPrivilegeValueW")))
#endif
	{
		goto cleanup;
	}

	/* Get the local id of our desired privilege */
	if (!lookupprivilegevalue(NULL, lpPrivilege, &tp.Privileges[0].Luid))
		goto cleanup;

	tp.PrivilegeCount           = 1;
	tp.Privileges[0].Attributes = (bEnable ? SE_PRIVILEGE_ENABLED : 0);

	/* Get the access token for the current process */
	if (!openprocesstoken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &hToken))
		goto cleanup;

	/* Enable the privilege */
	if (!adjusttokenprivileges(hToken, FALSE, &tp, sizeof(TOKEN_PRIVILEGES), NULL, NULL))
		goto cleanup;

	bResult = TRUE;

cleanup:
	if (hAdvapi) FreeLibrary(hAdvapi);
	if (hToken)  CloseHandle(hToken);
	return bResult;
}


#if 1 /* Test code. */
int main(void)
{
	EnableProcessPrivilege(SE_SHUTDOWN_NAME, TRUE);
	EnableProcessPrivilege(SE_SHUTDOWN_NAME, FALSE);

	if (!ExitWindowsEx(EWX_POWEROFF, 0))
		MessageBox(NULL, TEXT("ExitWindowsEx correctly failed."), NULL, 0);

	EnableProcessPrivilege(SE_SHUTDOWN_NAME, TRUE);

	if (ExitWindowsEx(EWX_POWEROFF, 0))
		MessageBox(NULL, TEXT("ExitWindowsEx correctly succeeded."), NULL, 0);

	return 0;
}
#endif